import type { Client, Command, MetadataBearer } from '@smithy/types';
/** A type identifying _any_ AWS {@link Client} */
type AnyClient = Client<any, any, any>;
/** A type identifying _any_ AWS {@link Command} */
type AnyCommand = Command<any, any, any, any, any>;
/** Infer the _input type_ of an AWS {@link Command} */
type CommandInput<T> = T extends Command<any, infer Input, any, any, any> ? Input : never;
/** Infer the _output type_ of an AWS {@link Command} */
type CommandOutput<T> = T extends Command<any, any, any, infer Output, any> ? Omit<Output, '$metadata'> & Partial<MetadataBearer> : never;
/** A type identifying a _constructor_ for an instance */
type Constructor<T> = new (...args: any) => T;
/** An interface defining a _call record_ in a {@link AWSMock} */
export interface AWSMockCall {
    command: string;
    input: any;
    success: boolean;
}
/** A _handler_ for intercepting commands in {@link AWSMock} */
export type AWSMockHandler<Input, Output, State = any> = (input: Input, state: State | undefined) => Output | Promise<Output>;
/** The {@link AWSMock} class allows easy mocking of AWS clients */
export declare class AWSMock<State = any> {
    private _client;
    /** All configured command handlers */
    private _handlers;
    /** The _original_ (unmocked) `send(...)` method from the client prototype */
    private _send;
    /** Current record of all calls handled by this instance */
    private _calls;
    /** Current state to be passed to handlers */
    private _state;
    /** Create a new {@link AWSMock} instance mocking the specified {@link Client} */
    constructor(client: Constructor<AnyClient>);
    /** Associate a _state_ with this instance, to be passed to handlers */
    setState(state: State | undefined): void;
    /** Return the list of calls executed by this instance */
    getCalls(): readonly AWSMockCall[];
    /**
     * Reset this instance by zeroing the {@link AWSMock.calls calls} and wiping
     * the {@link AWSMock.state state}.
     *
     * @returns The list of calls executed by this instance before reset.
     */
    reset(): AWSMockCall[];
    /** Destroy this instance, restoring the mocked methods */
    destroy(): void;
    /** Setup a _handler_ for a specific {@link Command} */
    on<AWSCommand extends AnyCommand>(command: new (...args: any) => AWSCommand, handler: AWSMockHandler<CommandInput<AWSCommand>, CommandOutput<AWSCommand>, State>): this;
}
export {};
