// index.ts
import { randomUUID } from "node:crypto";
var AWSMock = class {
  // Overload to hide private parameter
  constructor(_client) {
    this._client = _client;
    this._send = _client.prototype.send;
    const clientName = _client.name || /* coverage ignore next */
    "[Unknown Client]";
    _client.prototype.send = (_command, ..._args) => {
      const commandProto = Object.getPrototypeOf(_command);
      const commandConstuctor = commandProto.constructor;
      const commandName = commandConstuctor?.name || /* coverage ignore next */
      "[Unknown Command]";
      const lastArgument = _args.pop();
      const cb = typeof lastArgument === "function" ? lastArgument : void 0;
      const input = structuredClone(_command.input);
      const promise = new Promise((resolve, reject) => setImmediate(async () => {
        const handler = this._handlers.get(commandConstuctor);
        if (!handler) {
          const error = new Error(`No mock for "${clientName}.${commandName}"`);
          return reject(error);
        }
        try {
          const output = await handler(input, this._state);
          if (!output) {
            const error = new Error(`Mock for "${clientName}.${commandName}" returned no result`);
            return reject(error);
          }
          const result = structuredClone(output);
          if (!result.$metadata)
            result.$metadata = { httpStatusCode: 200 };
          if (!result.$metadata.requestId)
            result.$metadata.requestId = randomUUID();
          resolve(result);
        } catch (error) {
          reject(error);
        }
      })).then((result) => {
        this._calls.push({ command: commandName, input, success: true });
        return result;
      }, (error) => {
        this._calls.push({ command: commandName, input, success: false });
        throw error;
      });
      if (cb)
        promise.then((result) => cb(null, result), (error) => cb(error));
      return promise;
    };
  }
  /** All configured command handlers */
  _handlers = /* @__PURE__ */ new Map();
  /** The _original_ (unmocked) `send(...)` method from the client prototype */
  _send;
  /** Current record of all calls handled by this instance */
  _calls = [];
  /** Current state to be passed to handlers */
  _state = void 0;
  /** Associate a _state_ with this instance, to be passed to handlers */
  setState(state) {
    this._state = state;
  }
  /** Return the list of calls executed by this instance */
  getCalls() {
    return this._calls;
  }
  /**
   * Reset this instance by zeroing the {@link AWSMock.calls calls} and wiping
   * the {@link AWSMock.state state}.
   *
   * @returns The list of calls executed by this instance before reset.
   */
  reset() {
    const calls = this._calls;
    delete this._state;
    this._calls = [];
    return calls;
  }
  /** Destroy this instance, restoring the mocked methods */
  destroy() {
    this._client.prototype.send = this._send;
  }
  /** Setup a _handler_ for a specific {@link Command} */
  on(command, handler) {
    this._handlers.set(command, handler);
    return this;
  }
};
export {
  AWSMock
};
//# sourceMappingURL=index.mjs.map
